"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const int64_buffer_1 = require("int64-buffer");
const libra_web_account_1 = require("../libra-web-account");
const libra_web_net_1 = require("../libra-web-net");
class LibraTransaction {
    /**
     * Create a new Transaction
     *
     * @param program
     * @param gasConstraint
     * @param expirationTime
     * @param sendersAddress
     * @param sequenceNumber
     */
    constructor(program, gasConstraint, expirationTime, sendersAddress, sequenceNumber) {
        this.program = program;
        this.gasContraint = gasConstraint;
        this.expirationTime = new bignumber_js_1.default(expirationTime);
        this.sendersAddress = new libra_web_account_1.AccountAddress(sendersAddress);
        this.sequenceNumber = new bignumber_js_1.default(sequenceNumber);
    }
}
exports.LibraTransaction = LibraTransaction;
class LibraTransactionFactory {
    static createTransfer(senderAddress, receipientAddress, numCoins, gasConstraint = {
        gasUnitPrice: new bignumber_js_1.default('0'),
        maxGasAmount: new bignumber_js_1.default('10000'),
    }, expirationTime = '1661898073', sequenceNumber = '-1') {
        const program = new libra_web_net_1.Program();
        program.setCode(new Uint8Array([
            76,
            73,
            66,
            82,
            65,
            86,
            77,
            10,
            1,
            0,
            7,
            1,
            74,
            0,
            0,
            0,
            4,
            0,
            0,
            0,
            3,
            78,
            0,
            0,
            0,
            6,
            0,
            0,
            0,
            12,
            84,
            0,
            0,
            0,
            5,
            0,
            0,
            0,
            13,
            89,
            0,
            0,
            0,
            4,
            0,
            0,
            0,
            5,
            93,
            0,
            0,
            0,
            41,
            0,
            0,
            0,
            4,
            134,
            0,
            0,
            0,
            32,
            0,
            0,
            0,
            7,
            166,
            0,
            0,
            0,
            13,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            0,
            2,
            0,
            1,
            3,
            0,
            2,
            0,
            2,
            4,
            2,
            3,
            2,
            4,
            2,
            6,
            60,
            83,
            69,
            76,
            70,
            62,
            12,
            76,
            105,
            98,
            114,
            97,
            65,
            99,
            99,
            111,
            117,
            110,
            116,
            4,
            109,
            97,
            105,
            110,
            15,
            112,
            97,
            121,
            95,
            102,
            114,
            111,
            109,
            95,
            115,
            101,
            110,
            100,
            101,
            114,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            1,
            2,
            0,
            4,
            0,
            12,
            0,
            12,
            1,
            17,
            1,
            2,
        ]));
        const arg1 = new libra_web_net_1.TransactionArgument();
        arg1.setType(libra_web_net_1.TransactionArgument.ArgType.ADDRESS);
        arg1.setData(receipientAddress);
        const arg2 = new libra_web_net_1.TransactionArgument();
        arg2.setType(libra_web_net_1.TransactionArgument.ArgType.U64);
        arg2.setData(new Uint8Array(new int64_buffer_1.Uint64LE(numCoins.toNumber()).toArray()));
        program.setArgumentsList([arg1, arg2]);
        program.setModulesList([]);
        return new LibraTransaction(program, gasConstraint, expirationTime, senderAddress, sequenceNumber);
    }
}
exports.LibraTransactionFactory = LibraTransactionFactory;
